package com.bjlt.spider.config;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.jedis.JedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.JdkSerializationRedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;
import redis.clients.jedis.JedisPoolConfig;
import redis.clients.jedis.JedisShardInfo;

/**
 * Created by Scott on 2017/6/8.
 */
@Configuration
public class RedisConfig {
    @Value("${spring.redis.host}")
    protected String redisHost;
    @Value("${spring.redis.port}")
    protected Integer redisPort;
    @Value("${spring.redis.timeout}")
    protected Integer redisTimeOut;
    @Value("${spring.redis.pool.max-idle}")
    protected Integer redisMaxIdle;
    @Value("${spring.redis.pool.max-wait}")
    protected Integer redisMaxWait;
    @Value("${spring.redis.password}")
    protected String password;




    @Bean
    RedisTemplate redisTemplate() {
        RedisTemplate  redisTemplate = new RedisTemplate();
        JedisConnectionFactory jedisConnectionFactory = new JedisConnectionFactory();
        JedisPoolConfig jedisPoolConfig = new JedisPoolConfig();

        jedisPoolConfig.setMaxIdle(redisMaxIdle);
        jedisPoolConfig.setMaxWaitMillis(redisMaxWait);
        jedisConnectionFactory.setPoolConfig(jedisPoolConfig);
        JedisShardInfo jedisShardInfo = new JedisShardInfo(redisHost,redisPort);
        jedisShardInfo.setPassword(password);
        jedisConnectionFactory.setShardInfo(jedisShardInfo);
        redisTemplate.setConnectionFactory(jedisConnectionFactory);
        redisTemplate.setKeySerializer(new StringRedisSerializer());
        redisTemplate.setValueSerializer(new JdkSerializationRedisSerializer());
        return redisTemplate;
    }
}
