package com.bjlt.spider.controller;

import com.alibaba.fastjson.JSONObject;
import com.bjlt.spider.Service.ScExchangeRateService;
import com.bjlt.spider.Service.ScPolicyService;
import com.bjlt.spider.entity.*;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Maps;
import com.luyn.date.DatePattren;
import com.luyn.date.DateUtil;
import com.luyn.http.okhttp.OkHttpDownLoader;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


@Slf4j
public class ScPolicyRunable implements  Runnable{

	private ScPolicyService scPolicyService;

	private ScExchangeRateService scExchangeRateService;

	private  AirSourceLineVo airSourceLineVo;

	private  List<String> dateLists;

	private String currencyCode;
	private String BIGPOINT;

	private String AKWEBURL;

	private ObjectMapper objectMapper =new ObjectMapper();

	public ScPolicyRunable(ScPolicyService scPolicyService,
						   ScExchangeRateService scExchangeRateService,
						   AirSourceLineVo airSourceLineVo,
						   List<String> dateLists,
						   String currencyCode,String BIGPOINT,String AKWEBURL) {
		this.scPolicyService = scPolicyService;
		this.scExchangeRateService =scExchangeRateService;
		this.airSourceLineVo=airSourceLineVo;
		this.dateLists=dateLists;
		this.currencyCode=currencyCode;
		this.BIGPOINT=BIGPOINT;
		this.AKWEBURL=AKWEBURL;
	}

	/**
	 * 对比 官网价格 和 积分价格 以big 积分 价格 为标准
	 */
	@Override
	public void run() {

		String fromCity =airSourceLineVo.getFromCity();
		String toCity =airSourceLineVo.getToCity();
		Map<String,String> json  = new HashMap<>();
		//连续查询5天，都没有数据就直接跳过
		Map<String,Object> map = Maps.newHashMap();
		map.put("day",5);
		for (int j = 0; j <dateLists.size() ; j++) {
			log.info("查询日期为"+dateLists.get(j)+fromCity+toCity);
			try {
				json.put("airSource","1");
				json.put("fromDate",dateLists.get(j));
				json.put("tripType","1");
				json.put("adultNum","1");
				json.put("childNum","0");
				json.put("currency",currencyCode);
				json.put("formCity",fromCity);
				json.put("retDate","");
				json.put("toCity",toCity);
				SearchResponseEntity bigResponse=null;
				String bightml=null;
				if (map.get("day")!=null && Integer.parseInt(map.get("day").toString())!=0){
					bightml= OkHttpDownLoader.textBody(BIGPOINT).json(json).readTimeout(10, TimeUnit.MINUTES).writeTimeout(10,TimeUnit.MINUTES).execute().asString();
					log.info("积分采集返回数据{}",bightml);
					JSONObject bigjson = JSONObject.parseObject(bightml);

					if (bigjson.get("status").toString().equals("200")){
						bigResponse = objectMapper.readValue(bigjson.get("attachment").toString(), SearchResponseEntity.class);
					}else {
						log.info("big积分采集失败"+bigjson.toString());
					}
				}else {
					//循环五次了发信并没有找到航程信息
					j=j+5;
					j=j>dateLists.size()-1?dateLists.size()-1:j;
					map.put("day",5);
				}
				if(bigResponse!=null && bigResponse.getData()!=null&& !"".equals(bigResponse.getData()) && Integer.parseInt(map.get("day").toString())!=0){
					System.out.println(bightml);

					String akhtml = OkHttpDownLoader.textBody(AKWEBURL).json(json).readTimeout(10,TimeUnit.MINUTES).writeTimeout(10,TimeUnit.MINUTES).execute().asString();
					log.info("官网采集返回数据{}",akhtml);
					JSONObject jsonObject = JSONObject.parseObject(akhtml);

					if (jsonObject.get("status").toString().equals("200")){
						SearchResponseEntity akresponse = objectMapper.readValue(jsonObject.get("attachment").toString(), SearchResponseEntity.class);
						System.out.println(akhtml);
						//生成政策
						getPolicy(bigResponse,akresponse);
					}else {
						log.info("官网查询失败："+jsonObject.toString());
					}
				}else {
					int  day = Integer.parseInt(map.get("day").toString());
					map.put("day",day-=1);
					log.info(fromCity+"->"+toCity+"在"+dateLists.get(j)+"没有查询到可预订航班信息"+"剩余"+day+"次查询");
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	private void getPolicy(SearchResponseEntity bigsearchResponse, SearchResponseEntity aksearchResponse) {
		List<AirlineInfomation> bigfromSegments = bigsearchResponse.getFromSegments();

		List<AirlineInfomation> akfromSegments = aksearchResponse.getFromSegments();

		if (bigfromSegments!=null &&bigfromSegments.size()!=0){
			bigfromSegments.stream().forEach(t -> {
				List<ScExchangeRate> currency = scExchangeRateService.listScExchangeRate(ScExchangeRate.builder().exchangeCurrency("CNY").groupId("1").foreCurrency(t.getCurrency()).build());
				double currencyRate = Double.valueOf(currency.get(0).getFinalRate());
				log.info("币种" + t.getCurrency() + "获得的汇率为：" + currency.get(0).getFinalRate());
				//获取所有的套餐类型
				List<AirlinePriceInfo> bigAirlinePriceInfos = t.getAirlinePriceInfos();
				//获取所有的航班信息
				List<RouteInfoEntity> routeInfoEntities = t.getRouteInfoEntities();

				Set<String> collect = routeInfoEntities.stream().map(RouteInfoEntity::getFlightNumber).collect(Collectors.toSet());

				akfromSegments.stream().forEach(f -> {
					Set<String> collect1 = f.getRouteInfoEntities().stream().map(RouteInfoEntity::getFlightNumber).collect(Collectors.toSet());

					List<AirlinePriceInfo> akAirlinePriceInfos = f.getAirlinePriceInfos();
					//航班信息一致
					if (collect.equals(collect1)) {
						bigAirlinePriceInfos.forEach(k -> {
							//加50 以后与官网价格比较
							List<AirlinePriceInfo> collect2 = akAirlinePriceInfos.stream().filter(ff -> ff.getCabinclass().equals(k.getCabinclass())).collect(Collectors.toList());
							double bigPrice = k.getAdultPrice().add(k.getAdultTax()).doubleValue() * currencyRate + 50;

							List<AirlinePriceInfo> useAirLine = collect2.stream().filter(kk -> bigPrice < kk.getAdultPrice().add(kk.getChildPrice()).doubleValue() * currencyRate).collect(Collectors.toList());

							//积分+50 仍然比官网小 的套餐生成政策
							if (useAirLine != null && useAirLine.size() != 0) {
								double ak = useAirLine.get(0).getAdultPrice().add(useAirLine.get(0).getAdultTax()).doubleValue() * currencyRate;
								scPolicyService.insertSelective(ScPolicy.builder().fromCity(t.getFormCity()).toCity(t.getToCity()).fromDateStart(t.getFromDate()).fromDateEnd(t.getFromDate()).flightNumberLimitType(1).lowPrice(new BigDecimal(10)).upPrice(new BigDecimal(99999999)).cid("QUNA").adjustType("0").finalPrice(new BigDecimal(bigPrice)).fileNo("积分系统").addOrSubtractMoney("0").originalPrice(new BigDecimal(ak)).addTime(DateUtil.getFormatDate(new Date(), DatePattren.TIME_FORMAT)).airSourceGroupId(1).score(String.valueOf(bigPrice)).policySource("point").cabinScope(k.getCabinclass()).build());
								log.info("积分政策生成" + t.getFormCity() + "-->" + t.getToCity() + ":" + t.getFromDate());
							}
						});
					}
				});

			});
		}
	}
}
