package com.bjlt.spider.dao.impl;

import com.bjlt.spider.dao.AirSourceLineDao;
import com.bjlt.spider.entity.AirSourceLineVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * 表 sc_airsource_line
 */
@Slf4j
@Repository
public class AirSourceLineDaoImpl implements AirSourceLineDao {

    @Autowired
    private JdbcTemplate jdbcTemplate;

    /**
     * 分页查询sc_airsource_line表的所有数据
     *
     * @param airSourceLineVo
     * @return
     */
    @Override
    public List<AirSourceLineVo> listPageAirSourceLine(AirSourceLineVo airSourceLineVo) {
        StringBuffer stringBuffer = new StringBuffer();
        String select = "select id, air_source_id as airSourceID, from_city, from_city_name as fromCityZh, to_city, to_city_name as toCityZh, from_country, edit_user, update_time, add_time, status from sc_airsource_line ";
        stringBuffer.append(select);
        Map<String, Object> map = map(airSourceLineVo);
        String where = "where 1 = 1" + map.keySet().stream().map(t -> " and " + t + "=?").collect(Collectors.joining(" "));

        stringBuffer.append(where);
        if (airSourceLineVo.getPageNum()!=null && airSourceLineVo.getSize()!=null){
            String limit = " limit " + (airSourceLineVo.getPageNum()-1)*airSourceLineVo.getSize() + "," + airSourceLineVo.getSize();
            stringBuffer.append(limit);
        }
        RowMapper<AirSourceLineVo> rowMapper = new BeanPropertyRowMapper<AirSourceLineVo>(AirSourceLineVo.class);

        List<AirSourceLineVo> list = jdbcTemplate.query(stringBuffer.toString(), rowMapper, map.values().toArray());

        if (list.size() != 0) {
            list.get(0).setTotal(count(where, map.values().toArray()));
        }
        return list;
    }

    private Integer count(String where, Object[] objects) {
        String sql = "select count(1) from sc_airsource_line " + where;
        return jdbcTemplate.queryForObject(sql, Integer.class, objects);
    }

    private static Map<String, Object> map(AirSourceLineVo airSourceLineVo) {
        Map<String, Object> map = new HashMap();
        if (airSourceLineVo.getId() != null) {
            map.put("id", airSourceLineVo.getId());
        }
        if (airSourceLineVo.getAirSourceID() != null && !"".equals(airSourceLineVo.getAirSourceID()) ) {
            map.put("air_source_id", airSourceLineVo.getAirSourceID());
        }

        if (airSourceLineVo.getFromCity() != null && !"".equals(airSourceLineVo.getFromCity())) {
            map.put("from_city", airSourceLineVo.getFromCity());
        }
        if (airSourceLineVo.getToCity() != null && !"".equals(airSourceLineVo.getToCity())) {
            map.put("to_city", airSourceLineVo.getToCity());
        }
        if (airSourceLineVo.getFromCityZh() != null && !"".equals(airSourceLineVo.getFromCityZh())) {
            map.put("from_city_name", airSourceLineVo.getFromCityZh());
        }
        if (airSourceLineVo.getToCityZh() != null && !"".equals(airSourceLineVo.getToCityZh())) {
            map.put("to_city_name", airSourceLineVo.getToCityZh());
        }
        if (airSourceLineVo.getFromCountry() != null && !"".equals(airSourceLineVo.getFromCountry())) {
            map.put("from_country", airSourceLineVo.getFromCountry());
        }
        if (airSourceLineVo.getEditUser() != null && !"".equals(airSourceLineVo.getEditUser())) {
            map.put("edit_user", airSourceLineVo.getEditUser());
        }
        if (airSourceLineVo.getUpdateTime() != null && !"".equals(airSourceLineVo.getUpdateTime())) {
            map.put("update_time", airSourceLineVo.getUpdateTime());
        }
        if (airSourceLineVo.getAddTime() != null && !"".equals(airSourceLineVo.getAddTime())) {
            map.put("add_time", airSourceLineVo.getAddTime());
        }
        if (airSourceLineVo.getStatus() != null && !"".equals(airSourceLineVo.getStatus())) {
            map.put("status", airSourceLineVo.getStatus());
        }

        return map;
    }

    /**
     * 修改
     */
    @Override
    public void updateByPrimaryKeySelective(AirSourceLineVo airSourceLineVo) throws SQLException {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append("update sc_airsource_line set ");
        Map<String, Object> map = map(airSourceLineVo);
        String update = map.keySet().stream().map(t -> t + "=?").collect(Collectors.joining(","));
        stringBuffer.append(update);
        System.out.println(update);
        stringBuffer.append(" where id=" + airSourceLineVo.getId());
        System.out.println(stringBuffer.toString());
        jdbcTemplate.update(stringBuffer.toString(), map.values().toArray());
    }

    /**
     * 增加数据
     */
    @Override
    public void insertSelective(AirSourceLineVo airSourceLineVo) {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append("insert into sc_airsource_line (");
        Map<String, Object> map = map(airSourceLineVo);
        stringBuffer.append(String.join(",", map.keySet()));
        stringBuffer.append(") values (");
        List<String> wenhao = map.keySet().stream().map(t -> "?").collect(Collectors.toList());
        stringBuffer.append(String.join(",", wenhao));
        stringBuffer.append(")");
        System.out.println(stringBuffer.toString());
        try {
            jdbcTemplate.update(stringBuffer.toString(), map.values().toArray());
        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    /**
     * 删除
     */
    @Override
    public void deleteAirSourceLine(String id) {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append("update sc_airsource_line set  status = 0 ");
        stringBuffer.append(" where id=" + id);
        System.out.println(stringBuffer.toString());
        jdbcTemplate.update(stringBuffer.toString());

    }

    @Override
    public AirSourceLineVo selectById(String id) {
        StringBuffer sql = new StringBuffer();
        String select = "select id, air_source_id as airSourceID, from_city, from_city_name as fromCityZh, to_city, to_city_name as toCityZh, from_country, edit_user, update_time, add_time, status from sc_airsource_line ";
        sql.append(select);
        sql.append(" where 1= 1 and id = ");
        sql.append("" + id + "");
        RowMapper<AirSourceLineVo> rowMapper = new BeanPropertyRowMapper<>(AirSourceLineVo.class);
        List<AirSourceLineVo> airSourceLineVos = jdbcTemplate.query(sql.toString(), rowMapper);
        AirSourceLineVo airSourceLineVo = null;
        if (airSourceLineVos != null && airSourceLineVos.size() > 0) {
            airSourceLineVo = airSourceLineVos.get(0);
        }
        return airSourceLineVo;
    }

    @Override
    public List<AirSourceLineVo> listAirSourceLine(AirSourceLineVo airSourceLineVo) {

        StringBuffer stringBuffer = new StringBuffer();
        String select = "select id, air_source_id as airSourceID, from_city, from_city_name as fromCityZh, to_city, to_city_name as toCityZh, from_country, edit_user, update_time, add_time, status from sc_airsource_line ";
        stringBuffer.append(select);
        Map<String, Object> map = map(airSourceLineVo);
        String where = "where 1 = 1" + map.keySet().stream().map(t -> " and " + t + "=?").collect(Collectors.joining(" "));
        stringBuffer.append(where);
        RowMapper<AirSourceLineVo> rowMapper = new BeanPropertyRowMapper<AirSourceLineVo>(AirSourceLineVo.class);

        List<AirSourceLineVo> list = jdbcTemplate.query(stringBuffer.toString(), rowMapper, map.values().toArray());

        return list;
    }

}
